@@ -1,26 +1,22 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * This file is part of libgpiod.
  *
  * Copyright (C) 2017-2018 Bartosz Golaszewski <bartekgola@gmail.com>
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or (at
- * your option) any later version.
  */
 
 /* GPIO line test cases. */
 
-#include "gpiod-test.h"
-
 #include <errno.h>
 
+#include "gpiod-test.h"
+
 static void line_request_output(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line_0;
 	struct gpiod_line *line_1;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -30,10 +26,10 @@ static void line_request_output(void)
 	TEST_ASSERT_NOT_NULL(line_0);
 	TEST_ASSERT_NOT_NULL(line_1);
 
-	status = gpiod_line_request_output(line_0, TEST_CONSUMER, 0);
-	TEST_ASSERT_RET_OK(status);
-	status = gpiod_line_request_output(line_1, TEST_CONSUMER, 1);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_output(line_0, TEST_CONSUMER, 0);
+	TEST_ASSERT_RET_OK(rv);
+	rv = gpiod_line_request_output(line_1, TEST_CONSUMER, 1);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_EQ(gpiod_line_get_value(line_0), 0);
 	TEST_ASSERT_EQ(gpiod_line_get_value(line_1), 1);
@@ -44,9 +40,9 @@ TEST_DEFINE(line_request_output,
 
 static void line_request_already_requested(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -54,11 +50,11 @@ static void line_request_already_requested(void)
 	line = gpiod_chip_get_line(chip, 0);
 	TEST_ASSERT_NOT_NULL(line);
 
-	status = gpiod_line_request_input(line, TEST_CONSUMER);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_input(line, TEST_CONSUMER);
+	TEST_ASSERT_RET_OK(rv);
 
-	status = gpiod_line_request_input(line, TEST_CONSUMER);
-	TEST_ASSERT_NOTEQ(status, 0);
+	rv = gpiod_line_request_input(line, TEST_CONSUMER);
+	TEST_ASSERT_NOTEQ(rv, 0);
 	TEST_ASSERT_ERRNO_IS(EBUSY);
 }
 TEST_DEFINE(line_request_already_requested,
@@ -67,9 +63,9 @@ TEST_DEFINE(line_request_already_requested,
 
 static void line_consumer(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -79,8 +75,8 @@ static void line_consumer(void)
 
 	TEST_ASSERT_NULL(gpiod_line_consumer(line));
 
-	status = gpiod_line_request_input(line, TEST_CONSUMER);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_input(line, TEST_CONSUMER);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT(!gpiod_line_needs_update(line));
 	TEST_ASSERT_STR_EQ(gpiod_line_consumer(line), TEST_CONSUMER);
@@ -91,9 +87,9 @@ TEST_DEFINE(line_consumer,
 
 static void line_consumer_long_string(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -103,9 +99,9 @@ static void line_consumer_long_string(void)
 
 	TEST_ASSERT_NULL(gpiod_line_consumer(line));
 
-	status = gpiod_line_request_input(line,
-					  "consumer string over 32 characters long");
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_input(line,
+				      "consumer string over 32 characters long");
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT(!gpiod_line_needs_update(line));
 	TEST_ASSERT_STR_EQ(gpiod_line_consumer(line),
@@ -118,8 +114,8 @@ TEST_DEFINE(line_consumer_long_string,
 
 static void line_request_bulk_output(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipA = NULL;
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipB = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipA = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipB = NULL;
 	struct gpiod_line_bulk bulkB = GPIOD_LINE_BULK_INITIALIZER;
 	struct gpiod_line_bulk bulkA;
 	struct gpiod_line *lineA0;
@@ -131,7 +127,7 @@ static void line_request_bulk_output(void)
 	struct gpiod_line *lineB2;
 	struct gpiod_line *lineB3;
 	int valA[4], valB[4];
-	int status;
+	int rv;
 
 	chipA = gpiod_chip_open(test_chip_path(0));
 	chipB = gpiod_chip_open(test_chip_path(1));
@@ -171,28 +167,28 @@ static void line_request_bulk_output(void)
 	valA[1] = 0;
 	valA[2] = 0;
 	valA[3] = 1;
-	status = gpiod_line_request_bulk_output(&bulkA, TEST_CONSUMER, valA);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_bulk_output(&bulkA, TEST_CONSUMER, valA);
+	TEST_ASSERT_RET_OK(rv);
 
 	valB[0] = 0;
 	valB[1] = 1;
 	valB[2] = 0;
 	valB[3] = 1;
-	status = gpiod_line_request_bulk_output(&bulkB, TEST_CONSUMER, valB);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_bulk_output(&bulkB, TEST_CONSUMER, valB);
+	TEST_ASSERT_RET_OK(rv);
 
 	memset(valA, 0, sizeof(valA));
 	memset(valB, 0, sizeof(valB));
 
-	status = gpiod_line_get_value_bulk(&bulkA, valA);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_get_value_bulk(&bulkA, valA);
+	TEST_ASSERT_RET_OK(rv);
 	TEST_ASSERT_EQ(valA[0], 1);
 	TEST_ASSERT_EQ(valA[1], 0);
 	TEST_ASSERT_EQ(valA[2], 0);
 	TEST_ASSERT_EQ(valA[3], 1);
 
-	status = gpiod_line_get_value_bulk(&bulkB, valB);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_get_value_bulk(&bulkB, valB);
+	TEST_ASSERT_RET_OK(rv);
 	TEST_ASSERT_EQ(valB[0], 0);
 	TEST_ASSERT_EQ(valB[1], 1);
 	TEST_ASSERT_EQ(valB[2], 0);
@@ -204,15 +200,15 @@ TEST_DEFINE(line_request_bulk_output,
 
 static void line_request_bulk_different_chips(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipA = NULL;
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipB = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipA = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipB = NULL;
 	struct gpiod_line_request_config req;
 	struct gpiod_line_bulk bulk;
 	struct gpiod_line *lineA0;
 	struct gpiod_line *lineA1;
 	struct gpiod_line *lineB0;
 	struct gpiod_line *lineB1;
-	int status;
+	int rv;
 
 	chipA = gpiod_chip_open(test_chip_path(0));
 	chipB = gpiod_chip_open(test_chip_path(1));
@@ -239,19 +235,57 @@ static void line_request_bulk_different_chips(void)
 	req.request_type = GPIOD_LINE_REQUEST_DIRECTION_INPUT;
 	req.flags = GPIOD_LINE_ACTIVE_STATE_HIGH;
 
-	status = gpiod_line_request_bulk(&bulk, &req, NULL);
-	TEST_ASSERT_NOTEQ(status, 0);
+	rv = gpiod_line_request_bulk(&bulk, &req, NULL);
+	TEST_ASSERT_NOTEQ(rv, 0);
 	TEST_ASSERT_ERRNO_IS(EINVAL);
 }
 TEST_DEFINE(line_request_bulk_different_chips,
 	    "gpiod_line_request_bulk() - different chips",
 	    0, { 8, 8 });
 
+static void line_request_null_default_vals_for_output(void)
+{
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
+	struct gpiod_line_bulk bulk = GPIOD_LINE_BULK_INITIALIZER;
+	struct gpiod_line *line;
+	int rv, vals[3];
+
+	chip = gpiod_chip_open(test_chip_path(0));
+	TEST_ASSERT_NOT_NULL(chip);
+
+	line = gpiod_chip_get_line(chip, 0);
+	gpiod_line_bulk_add(&bulk, line);
+	line = gpiod_chip_get_line(chip, 1);
+	gpiod_line_bulk_add(&bulk, line);
+	line = gpiod_chip_get_line(chip, 2);
+	gpiod_line_bulk_add(&bulk, line);
+
+	rv = gpiod_line_request_bulk_output(&bulk, TEST_CONSUMER, NULL);
+	TEST_ASSERT_RET_OK(rv);
+
+	gpiod_line_release_bulk(&bulk);
+
+	rv = gpiod_line_request_bulk_input(&bulk, TEST_CONSUMER);
+	TEST_ASSERT_RET_OK(rv);
+
+	memset(vals, 0, sizeof(vals));
+
+	rv = gpiod_line_get_value_bulk(&bulk, vals);
+	TEST_ASSERT_RET_OK(rv);
+
+	TEST_ASSERT_EQ(vals[0], 0);
+	TEST_ASSERT_EQ(vals[1], 0);
+	TEST_ASSERT_EQ(vals[2], 0);
+}
+TEST_DEFINE(line_request_null_default_vals_for_output,
+	    "gpiod_line_request_bulk() - null default vals for output",
+	    0, { 8 });
+
 static void line_set_value(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -259,8 +293,8 @@ static void line_set_value(void)
 	line = gpiod_chip_get_line(chip, 2);
 	TEST_ASSERT_NOT_NULL(line);
 
-	status = gpiod_line_request_output(line, TEST_CONSUMER, 0);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_output(line, TEST_CONSUMER, 0);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_RET_OK(gpiod_line_set_value(line, 1));
 	TEST_ASSERT_EQ(gpiod_line_get_value(line), 1);
@@ -273,8 +307,8 @@ TEST_DEFINE(line_set_value,
 
 static void line_get_value_different_chips(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipA = NULL;
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chipB = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipA = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chipB = NULL;
 	struct gpiod_line *lineA1, *lineA2, *lineB1, *lineB2;
 	struct gpiod_line_bulk bulkA, bulkB, bulk;
 	int rv, vals[4];
@@ -389,9 +423,9 @@ TEST_DEFINE(line_find_unnamed_lines,
 
 static void line_direction(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -399,15 +433,15 @@ static void line_direction(void)
 	line = gpiod_chip_get_line(chip, 5);
 	TEST_ASSERT_NOT_NULL(line);
 
-	status = gpiod_line_request_output(line, TEST_CONSUMER, 0);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_output(line, TEST_CONSUMER, 0);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_EQ(gpiod_line_direction(line), GPIOD_LINE_DIRECTION_OUTPUT);
 
 	gpiod_line_release(line);
 
-	status = gpiod_line_request_input(line, TEST_CONSUMER);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_input(line, TEST_CONSUMER);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_EQ(gpiod_line_direction(line), GPIOD_LINE_DIRECTION_INPUT);
 }
@@ -417,9 +451,9 @@ TEST_DEFINE(line_direction,
 
 static void line_active_state(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -427,17 +461,17 @@ static void line_active_state(void)
 	line = gpiod_chip_get_line(chip, 5);
 	TEST_ASSERT_NOT_NULL(line);
 
-	status = gpiod_line_request_input(line, TEST_CONSUMER);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request_input(line, TEST_CONSUMER);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_EQ(gpiod_line_active_state(line),
 		       GPIOD_LINE_ACTIVE_STATE_HIGH);
 
 	gpiod_line_release(line);
 
-	status = gpiod_line_request_input_flags(line, TEST_CONSUMER,
+	rv = gpiod_line_request_input_flags(line, TEST_CONSUMER,
 					GPIOD_LINE_REQUEST_FLAG_ACTIVE_LOW);
-	TEST_ASSERT_RET_OK(status);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT_EQ(gpiod_line_direction(line), GPIOD_LINE_DIRECTION_INPUT);
 }
@@ -447,10 +481,10 @@ TEST_DEFINE(line_active_state,
 
 static void line_misc_flags(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line_request_config config;
 	struct gpiod_line *line;
-	int status;
+	int rv;
 
 	chip = gpiod_chip_open(test_chip_path(0));
 	TEST_ASSERT_NOT_NULL(chip);
@@ -466,8 +500,8 @@ static void line_misc_flags(void)
 	config.consumer = TEST_CONSUMER;
 	config.flags = GPIOD_LINE_REQUEST_FLAG_OPEN_DRAIN;
 
-	status = gpiod_line_request(line, &config, 0);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request(line, &config, 0);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT(gpiod_line_is_used(line));
 	TEST_ASSERT(gpiod_line_is_open_drain(line));
@@ -477,8 +511,8 @@ static void line_misc_flags(void)
 
 	config.flags = GPIOD_LINE_REQUEST_FLAG_OPEN_SOURCE;
 
-	status = gpiod_line_request(line, &config, 0);
-	TEST_ASSERT_RET_OK(status);
+	rv = gpiod_line_request(line, &config, 0);
+	TEST_ASSERT_RET_OK(rv);
 
 	TEST_ASSERT(gpiod_line_is_used(line));
 	TEST_ASSERT_FALSE(gpiod_line_is_open_drain(line));
@@ -490,7 +524,7 @@ TEST_DEFINE(line_misc_flags,
 
 static void line_open_source_open_drain_input_invalid(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
 	int rv;
 
@@ -516,7 +550,7 @@ TEST_DEFINE(line_open_source_open_drain_input_invalid,
 
 static void line_open_source_open_drain_simultaneously(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line *line;
 	int rv;
 
@@ -536,9 +570,48 @@ TEST_DEFINE(line_open_source_open_drain_simultaneously,
 	    "gpiod_line - open-source & open-drain flags simultaneously",
 	    0, { 8 });
 
+/* Verify that the reference counting of the line fd handle works correctly. */
+static void line_release_one_use_another(void)
+{
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
+	struct gpiod_line_bulk bulk;
+	struct gpiod_line *line1;
+	struct gpiod_line *line2;
+	int rv, vals[2];
+
+	chip = gpiod_chip_open(test_chip_path(0));
+	TEST_ASSERT_NOT_NULL(chip);
+
+	line1 = gpiod_chip_get_line(chip, 2);
+	TEST_ASSERT_NOT_NULL(line1);
+	line2 = gpiod_chip_get_line(chip, 3);
+	TEST_ASSERT_NOT_NULL(line2);
+
+	gpiod_line_bulk_init(&bulk);
+	gpiod_line_bulk_add(&bulk, line1);
+	gpiod_line_bulk_add(&bulk, line2);
+
+	vals[0] = vals[1] = 1;
+
+	rv = gpiod_line_request_bulk_output(&bulk, TEST_CONSUMER, vals);
+	TEST_ASSERT_RET_OK(rv);
+
+	gpiod_line_release(line1);
+
+	rv = gpiod_line_get_value(line1);
+	TEST_ASSERT_EQ(rv, -1);
+	TEST_ASSERT_ERRNO_IS(EPERM);
+
+	rv = gpiod_line_get_value(line2);
+	TEST_ASSERT_EQ(rv, 1);
+}
+TEST_DEFINE(line_release_one_use_another,
+	    "gpiod_line - request two, release one, use the other one",
+	    0, { 8 });
+
 static void line_null_consumer(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line_request_config config;
 	struct gpiod_line *line;
 	int rv;
@@ -575,7 +648,7 @@ TEST_DEFINE(line_null_consumer,
 
 static void line_empty_consumer(void)
 {
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line_request_config config;
 	struct gpiod_line *line;
 	int rv;
@@ -617,7 +690,7 @@ static void line_bulk_foreach(void)
 						  "gpio-mockup-A-2",
 						  "gpio-mockup-A-3" };
 
-	TEST_CLEANUP(test_close_chip) struct gpiod_chip *chip = NULL;
+	TEST_CLEANUP_CHIP struct gpiod_chip *chip = NULL;
 	struct gpiod_line_bulk bulk = GPIOD_LINE_BULK_INITIALIZER;
 	struct gpiod_line *line, **lineptr;
 	int i;
